//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                          //
//                                          Marblevator, Cradle                                             //
//                                                                                                          //
//                                          Copyright © 2022 by                                             //
//                                        Zumwalt Properties, LLC.                                          //
//                                          All Rights Reserved.                                            //
//                                                                                                          //
//          This software is for NON COMMERCIAL USE ONLY in accordance with the licensing agreement         //
//                                     as published on this website.                                        //
//                                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char*     chCopyrightStart = "Marblevator, Cradle.  Copyright © 2022 by Zumwalt Properties, LLC.\
                                    All Rights Reserved.";
  
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Non 3D printed components.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// One Arduino Nano 33 IoT.
// One 11.1vdc 300mAh 3S 30c LiPo battery.
// One JST mating connector for the battery.
// One Gikfun EK1909 "magnetic levitation coil".
// Two TCRT5000 infrared transceivers.
// One FQP30N06L mosfet.
// One 1N4007 diode.
// One 10KΩ 1/8 watt resistor.
// One 2.7kΩ 1/8 watt resistor.
// Two 1KΩ 1/8 watt resistors.
// One M3 by 8mm cap screw.
// One 11mm ball bearing.
// One M3 by 8mm cap screw.

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// 3D printed components.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// 1) One "Base.stl" with PLA support.
// 2) Two "Bolt, Mount, Coil.stl".
// 3) One "Bottom.stl".
// 4) One "Mount, Coil.stl".
// 5) One "Track.stl" with breakaway or PVA support (requires a dual extrusion 3D printer).

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Included Libraries and/or Files.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <ArduinoLowPower.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Constants.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define                 COIL_ON_DELAY_MICROSENDS                   4000
#define                 COIL_ON_MICROSECONDS                       1600
#define                 INTERRUPT_PIN_IRXCVR_1                     9
#define                 INTERRUPT_PIN_IRXCVR_2                     10
#define                 MOSFET_PIN                                 3
#define                 BATTERY_VOLTAGE_PIN                        A7

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Global variables.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

volatile  byte          bLastInfraredTransceiverSensorInterrupt    = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// setup().
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void setup() 
{ 
  // Disable interrupts.
  
  noInterrupts();

  // Set the mosfet drive pin as an output.

  pinMode(MOSFET_PIN, OUTPUT);
  digitalWrite(MOSFET_PIN, 0);

  // Set the infrared transceiver 1 interrupt sense pin as an input, then assign the interrupt service routine for it.

  pinMode(INTERRUPT_PIN_IRXCVR_1, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN_IRXCVR_1), InterruptServiceInfraredTransceiver1, LOW);
  
  // Set the infrared transceiver 2 interrupt sense pin as an input, then assign the interrupt service routine for it.

  pinMode(INTERRUPT_PIN_IRXCVR_2, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN_IRXCVR_2), InterruptServiceInfraredTransceiver2, LOW);

  // Set the led output pin.

  pinMode(LED_BUILTIN, OUTPUT);

  // With setup complete, enable interrupts.

  interrupts();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// loop().
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void loop() 
{
  
  // Local variables.

    // Declare the model battery level parameters.
    
    static  float fAdInputMaxVoltageMax       = 3.3;                                                                  // the Arduino max ad input voltage
    static  float fAdMaxCounts                = 1024.0;                                                               // the Arduino max ad count (0 through 1023 = 1024 counts representing 0 throu 3.3vDC)
    static  float fCellCount                  = 3.0;                                                                  // the number of LiPo cells in the specified battery pack
    static  float fCellCutoffVoltage          = 3.2;                                                                  // the cutoff voltage of the individual cells in the battery pack
    static  float fResistorDividerLowerInOhms = 2700.0;                                                               // the lower resistor value in ohms of the resistor divider used in determining the battery pack voltage
    static  float fResistorDividerUpperInOhms = 10000.0;                                                              // the upper resistor value in ohms of the resistor divider used in determining the battery pack voltage

    // Using the model battery level parameters, calculate the A/D count that represents the low voltage cutoff for the LiPo battery.
    
    static  int   nBatteryLowVoltageADCount   = (int)(((((fCellCutoffVoltage * fCellCount) /                          // battery pack cutoff voltage = 9.6VDC
                                                     (fResistorDividerUpperInOhms + fResistorDividerLowerInOhms)) *   // divided by the resistor divider from top to bottom is 9.6vDC / 12700 = total divider current of .7559mA
                                                     fResistorDividerLowerInOhms) /                                   // multiplied by the lower resistor value fResistorDividerLowerInOhms * .7559mA = cutoff voltage at ad input of 2.04093vDC
                                                     fAdInputMaxVoltageMax) *                                         // divided by the ad max input voltage - (2.04093 / 3.3) = .6184636
                                                     fAdMaxCounts);                                                   // multiplied by the max ad count =  (6184636 * 1024) = 633.30672, thus 633 is the low voltage battery cutoff ad counts

    // Delay 10 seconds before checking the battery voltage.

      delay(10000);
    
    // Check the battery voltage
  
      // Read the battery voltage.
      
      int adInput = analogRead(BATTERY_VOLTAGE_PIN);
  
      // Compare the battery voltage ad count with the cutoff ad count.
      
      if(adInput < nBatteryLowVoltageADCount)
      {
        // Battery voltage ad count is below the cutoff ad count, go to sleep.

        LowPower.deepSleep();
      }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// InterruptServiceInfraredTransceiver1().
//
// Infrared transceiver 1 interrupt service.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void  InterruptServiceInfraredTransceiver1()
{
  switch(bLastInfraredTransceiverSensorInterrupt)
  {
    case  0:
    case  1:
    {
      // If none (0) or 1, pulse the coil as the marble is rolling downhill.

        // Delay.

        delayMicroseconds(COIL_ON_DELAY_MICROSENDS);
        
        // Wait for INTERRUPT_PIN_IRXCVR_1 to return high.

        while(!digitalRead(INTERRUPT_PIN_IRXCVR_1))
        {}

        // Pulse the coil for COIL_ON_MICROSECONDS.

        digitalWrite(MOSFET_PIN, 1);
        delayMicroseconds(COIL_ON_MICROSECONDS);
        digitalWrite(MOSFET_PIN, 0);

        // Turn on the led.

        digitalWrite(LED_BUILTIN, HIGH);
    }
    break;
  }
  
  // Whether the coil was pulsed or not, infrared transceiver 1 is now the last interrupt source.

  bLastInfraredTransceiverSensorInterrupt = 1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// InterruptServiceInfraredTransceiver2().
//
// Infrared transceiver 2 interrupt service.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void  InterruptServiceInfraredTransceiver2()
{
  // Check which interrupt happened previous to this one.

  switch(bLastInfraredTransceiverSensorInterrupt)
  {
    case  0:
    case  2:
    {
      // If none (0) or 2, pulse the coil as the marble is rolling downhill.

        // Delay.

        delayMicroseconds(COIL_ON_DELAY_MICROSENDS);
        
        // Wait for INTERRUPT_PIN_IRXCVR_2 to return high.

        while(!digitalRead(INTERRUPT_PIN_IRXCVR_2))
        {}
        
        // Pulse the coil for COIL_ON_MICROSECONDS.

        digitalWrite(MOSFET_PIN, 1);
        delayMicroseconds(COIL_ON_MICROSECONDS);
        digitalWrite(MOSFET_PIN, 0);
 
        // Turn off the led.

        digitalWrite(LED_BUILTIN, LOW);
   }
    break;
  }

  // Whether the coil was pulsed or not, infrared transceiver 2 is now the last interrupt source.

  bLastInfraredTransceiverSensorInterrupt = 2;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char*     chCopyrightEnd = "Marblevator, Cradle.  Copyright © 2022 by Zumwalt Properties, LLC.\
                                  All Rights Reserved.";
  
